/*
ESP8266 MQTT Motion and Battery
v 1.1.8 

A low power motion sensor with battery voltage reporting MQTT. 
This code was designed for use with Home Assistant and MQTT. 
Due to the low power needs and sleep capabilities of ESP8266 a simple 
deep sleep wake circuit is used to trigger reporting motion and the
status of the battery.

Further details are available in the link to the Instructables
Insert link here

created 21 April 2020
by Michael Fong

*/
#include <hp_BH1750.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

#define NB_TRYWIFI 10

// load variables for network and MQTT server here
const char* ssid = "Your Wifi Network";
const char* password = "Your Wifi Password";
const char* mqtt_server = "Your MQTT server IP here";
const char* mqtt_password = "Your MQTT Password here";
const char* mqtt_username = "Name of your MQTT Broker here";

// load your MQTT topics and payload here
const char* mqtt_topic1 = "Sensor/stairs/motion"; //my current default, change to your MQTT preferences
const char* mqtt_payload1 = "on";
const char* mqtt_topic2 = "Sensor/stairs/battery"; //for example Motion/livingroom/battery
const char* mqtt_payload2 = "off";

//load values for pins here
const int analogInPin = A0;
const int motionPin = 5;
int sensorValue = 0;
int batteryValue = 0;

//timer variables
unsigned long now = millis();


WiFiClient espClient;
PubSubClient client(espClient);

void sendMQTTMessage(){
  if (!client.connected()) {
    reconnect();
  }
  client.publish(mqtt_topic1,mqtt_payload1);
  delay(100);
  client.publish(mqtt_topic2,String(batteryValue).c_str(),true); //sends battery level
  delay(100);
}

void setup() {
  // put your setup code here, to run once:
  pinMode(motionPin, INPUT);  
  Serial.begin(115200);

  // use a static IP here for best results
  WiFi.begin(ssid, password);
  IPAddress ip(192,168,68,206); //set your static IP for this sensor
  IPAddress gateway(192,168,68,1);
  IPAddress subnet(255,255,255,0);
  WiFi.config(ip,gateway,subnet);

  //Serial.println("connecting to Wifi");
  int _try = 0;
  while (WiFi.status() != WL_CONNECTED) {
    //Serial.print(".");
    delay(500);
    _try++;
    if ( _try >= NB_TRYWIFI ) {
      //Serial.println("Impossible to connect Wifi network, go to deep sleep");
      ESP.deepSleep(0);
    }
  }
  //Serial.println("Connected to WiFi network");
  
  //read battery levels here (3.7v Lipo in the design w/ 120K resistor, change to your setup)
  sensorValue = analogRead(analogInPin);
  batteryValue = map(sensorValue, 762, 976, 0, 100);
  //Serial.println(batteryValue); //uncomment for debugging
  
  //Serial.print("\t output = "); //uncomment for debugging
  //Serial.println(sensorValue); //uncomment for debugging
  
  //Serial.println("Send PIR Status to MQTT Broker");
  client.setServer(mqtt_server, 1883);
  sendMQTTMessage();
  delay(100);

  
}

void loop() {
  // put your main code here, to run repeatedly:
  if (digitalRead(motionPin) < 1){
    client.publish(mqtt_topic1,mqtt_payload2);
    delay(100);
    ESP.deepSleep(0);
  }
  //Serial.println("waking time: ");
  //Serial.print(millis()-now);
  //Serial.println("ms");
  //Serial.println(digitalRead(motionPin));
  //client.publish(mqtt_topic1,mqtt_payload2);
  //delay(100);
  //ESP.deepSleep(0);
 
}

boolean reconnect() {
  if (client.connect("ESP8266Client",mqtt_username,mqtt_password)){
    //Serial.println("connected to server");
    return client.connected();
  }
  //Serial.println("I think connection failed!");
  return 0;
}
